library angular2.src.core.compiler.pipeline.shim_shadow_css_dart;

import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank, Type;

class ShimShadowCss extends CompileStep {
  ShadowDomStrategy _strategy;
  Element _styleHost;
  Element _lastInsertedStyle;
  Type _component;
  ShimShadowCss(DirectiveMetadata cmpMetadata, ShadowDomStrategy strategy,
      Element styleHost)
      : super() {
    this._strategy = strategy;
    this._component = cmpMetadata.type;
    this._styleHost = styleHost;
    this._lastInsertedStyle = null;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (DOM.tagName(current.element) == 'STYLE') {
      current.ignoreBindings = true;
      if (this._strategy.extractStyles()) {
        DOM.remove(current.element);
        var css = DOM.getText(current.element);
        var shimComponent = this._strategy.getShimComponent(this._component);
        css = shimComponent.shimCssText(css);
        this._insertStyle(this._styleHost, css);
      }
    }
  }
  _insertStyle(Element el, String css) {
    var style = DOM.createStyleElement(css);
    if (isBlank(this._lastInsertedStyle)) {
      var firstChild = DOM.firstChild(el);
      if (isPresent(firstChild)) {
        DOM.insertBefore(firstChild, style);
      } else {
        DOM.appendChild(el, style);
      }
    } else {
      DOM.insertAfter(this._lastInsertedStyle, style);
    }
    this._lastInsertedStyle = style;
  }
}
