library angular2.test.core.compiler.pipeline.shim_shadow_css_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, expect, it, iit, ddescribe, el;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/shim_shadow_css.dart'
    show ShimShadowCss;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/shim_component.dart'
    show ShimComponent;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/facade/lang.dart' show Type;

main() {
  describe('ShimShadowCss', () {
    createPipeline(ShadowDomStrategy strategy, styleHost) {
      var component = new Component(selector: 'selector');
      var meta = new DirectiveMetadata(null, component);
      var shimShadowCss = new ShimShadowCss(meta, strategy, styleHost);
      return new CompilePipeline([shimShadowCss]);
    }
    it('it should set ignoreBindings to true for style elements', () {
      var host = el('<div></div>');
      var pipeline = createPipeline(new FakeStrategy(false), host);
      var results = pipeline.process(el('<div><style></style></div>'));
      expect(results[0].ignoreBindings).toBe(false);
      expect(results[1].ignoreBindings).toBe(true);
    });
    it('should not extract the styles when extractStyles() is false', () {
      var host = el('<div></div>');
      var pipeline = createPipeline(new FakeStrategy(false), host);
      var template = el('<style>.s{}</style>');
      pipeline.process(template);
      expect(template).toHaveText('.s{}');
    });
    it('should move the styles to the host when extractStyles() is true', () {
      var host = el('<div></div>');
      var pipeline = createPipeline(new FakeStrategy(true), host);
      var template = el('<div><style>.s{}</style></div>');
      pipeline.process(template);
      expect(template).toHaveText('');
      expect(host).toHaveText('/* shim */.s{}');
    });
    it('should preserve original content when moving styles', () {
      var host = el('<div>original content</div>');
      var pipeline = createPipeline(new FakeStrategy(true), host);
      var template = el('<div><style>.s{}</style></div>');
      pipeline.process(template);
      expect(template).toHaveText('');
      expect(host).toHaveText('/* shim */.s{}original content');
    });
    it('should move the styles to the host in the original order', () {
      var host = el('<div></div>');
      var pipeline = createPipeline(new FakeStrategy(true), host);
      var template = el('<div><style>.s1{}</style><style>.s2{}</style></div>');
      pipeline.process(template);
      expect(host).toHaveText('/* shim */.s1{}/* shim */.s2{}');
    });
  });
}
class FakeStrategy extends ShadowDomStrategy {
  bool _extractStyles;
  FakeStrategy(bool extractStyles) : super() {
    this._extractStyles = extractStyles;
  }
  bool extractStyles() {
    return this._extractStyles;
  }
  ShimComponent getShimComponent(Type component) {
    return new FakeShimComponent(component);
  }
}
class FakeShimComponent extends ShimComponent {
  FakeShimComponent(Type component) : super(component) {}
  String shimCssText(String cssText) {
    return '/* shim */' + cssText;
  }
}
