library angular2.test.core.compiler.pipeline.shim_shadow_dom_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, expect, it, iit, ddescribe, el;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/shim_shadow_dom.dart'
    show ShimShadowDom;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/shim_component.dart'
    show ShimComponent;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/facade/lang.dart' show Type, isBlank;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;

main() {
  describe('ShimShadowDom', () {
    createPipeline(bool ignoreBindings) {
      var component = new Component(selector: 'selector');
      var meta = new DirectiveMetadata(null, component);
      var shimShadowDom = new ShimShadowDom(meta, new FakeStrategy());
      return new CompilePipeline([
        new MockStep((parent, current, control) {
          current.ignoreBindings = ignoreBindings;
        }),
        new MockStep((parent, current, control) {
          var el = current.element;
          if (DOM.hasClass(el, 'host')) {
            current.componentDirective =
                new DirectiveMetadata(SomeComponent, null);
          }
        }),
        shimShadowDom
      ]);
    }
    it('should add the content attribute to content element', () {
      var pipeline = createPipeline(false);
      var results = pipeline.process(el('<div></div>'));
      expect(DOM.getAttribute(results[0].element, '_ngcontent'))
          .toEqual('content');
      expect(isBlank(DOM.getAttribute(results[0].element, '_nghost')))
          .toBeTruthy();
    });
    it('should add both the content and host attributes to host element', () {
      var pipeline = createPipeline(false);
      var results = pipeline.process(el('<div class="host"></div>'));
      expect(DOM.getAttribute(results[0].element, '_ngcontent'))
          .toEqual('content');
      expect(DOM.getAttribute(results[0].element, '_nghost')).toEqual('host');
    });
    it('should do nothing when ignoreBindings is true', () {
      var pipeline = createPipeline(true);
      var results = pipeline.process(el('<div class="host"></div>'));
      expect(isBlank(DOM.getAttribute(results[0].element, '_ngcontent')))
          .toBeTruthy();
      expect(isBlank(DOM.getAttribute(results[0].element, '_nghost')))
          .toBeTruthy();
    });
  });
}
class FakeStrategy extends ShadowDomStrategy {
  FakeStrategy() : super() {}
  ShimComponent getShimComponent(Type component) {
    return new FakeShimComponent(component);
  }
}
class FakeShimComponent extends ShimComponent {
  FakeShimComponent(Type component) : super(component) {}
  shimContentElement(Element element) {
    DOM.setAttribute(element, '_ngcontent', 'content');
  }
  shimHostElement(Element element) {
    DOM.setAttribute(element, '_nghost', 'host');
  }
}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
class SomeComponent {}
